#####   FASS_Base_Preprocess.R
#####

library(foreign)
library(readxl)

Read_Datafile <- function (Filename, Filepath, FileLocation)
{
  
  ### Parse file type
  SplitName <- strsplit(Filename, split = "[.]")
  
  Extension <- SplitName[[1]][2]
  BaseName <- SplitName[[1]][1]
  
  print(paste0("Loading data from file: ", Filename))
  
  ### STATA data format
  if (Extension == "dta")
  {
    Data <- read.dta(paste0(FileLocation, Filepath, Filename))
  }
  
  ### SPSS data format
  if (Extension == "sav")
  {
    Data <- read.spss(paste0(FileLocation, Filepath, Filename), use.value.labels=TRUE)
    
    Data <- as.data.frame(Data)
  }
  
  ### CSV format
  if (Extension == "csv")
  {
    Data <- read.csv(paste0(FileLocation, Filepath, Filename), header = TRUE)
  }
  
  ###Excel format
  if (Extension == "xls" || Extension == "xlsx")
  {
    Data <- read_excel(paste0(FileLocation, Filepath, Filename))
    
  }
  
  
  
  return(Data)
  
}

#####   FUNCTION: Write_Datafile
###       Writes the processed data into a data file. The name of an already existing datafile (containing the non-processed data) is provided,
#           in order to allow the new filename to be an extension of the old one. If neither ExtendedName nor FileType are provided, the old data
#           file will be overloaded.
#         Arguments:
#           PrevFilename  - Name of the file containing the non-processed input data. Data won't be manipulated AT ALL.
#           Location      - Path to data location
#           ExtendedName  - String to be concatenated at the end of the name provided into PrevFilename (filetype will be pasted later). If not
#                             provided, name will be the same.
#           FileType      - Extension for datafile. Must be a legal R, SPSS, STATA or xls extension. If not provided, extension from PrevFilename
#                             will be used.
#           DataSet       - Post-processed data to be written.

Write_Datafile <- function(PrevFilename, Location, ExtendedName = NULL, FileType = NULL, DataSet)
{
  
  SplitName <- strsplit(PrevFilename, split = "[.]")
  
  if (is.null(FileType))
  {
    Type <- SplitName[[1]][2]
  }
  else
  {
    Type <- FileType
  }
  
  BaseName <- SplitName[[1]][1]
  
  if (is.null(ExtendedName )) {
    FullFilename <- paste0(Location, BaseName, ".", Type)  
  }
  else {
    FullFilename <- paste0(Location, BaseName, ExtendedName, ".", Type)
  }
  
  
  ### R data formats
  if (Type == "RData" || Type == "Rdata" || Type == "Rdat")
  {
    print("NOT IMPLEMENTED YET: Export to .dta or .sav extension")
  }
  
  ### STATA data format
  if (Type == "dta")
  {
    
    print(paste0("Exporting data to file ", FullFilename))
    write_dta(DataSet, FullFilename)
  }
  
  ### SPSS data format
  if (Type == "sav")
  {
    write.foreign(DataSet, FullFilename, package="SPSS") 
  }
  
  if (Type == "xls" || Type == "xlsx")
  {
    print(paste0("Exporting data to file ", FullFilename))
    library(xlsx)
    write.xlsx(DataSet, FullFilename) 
  }
  
  if (Type == "csv")
  {
    print(paste0("Exporting data to file ", FullFilename))
    write.csv(DataSet, FullFilename, row.names = FALSE)
  }
  
}

Print_TXT <- function (Filename, Values, Titles)
{
  sink(Filename)
  for (i in 1:length(Values))
  {
    print(Titles[i])
    print(Values[[i]])
    writeLines("\n")
  }
  sink()
  
  
}


Print_Graph <- function (Filename, Values, Title, AllTogether)
{
  
  pdf(Filename)
  
  for (Value in Values)
  {
    
    XValue <- Value[1]
    YValue <- Value[2]
    
    plot(x = XValue, y = YValue, main = Title)
    
    if (AllTogether)
    {
      par(new = TRUE)
    }
    
  }
  
  dev.off
  
}

Read_DB <- function()
{
  Database <- Read_Datafile( Filepath = DataFolder, Filename = db_names, FileLocation = ProjectLocation )
  return(Database)
  }

#####   Function: PreprocessDB
###
##
PreprocessDB <- function ( DataSet, 
                           Symptom_Vars = NULL)
{
  
  ###   Error control: DataSet necessary for preprocess  
  if (is.null(DataSet))
  {
    print("Error in PreprocessDB (FASS_Base_Preprocess.R): Database not defined")
  }
  
  
  #####   SYMPTOMS PREPROCESS
  ###     Symptoms to numeric values (0,1, NA).
  #       Only if data is not background data
  if (!is.null(Symptom_Vars))
  {
    DataSet <- PreprocessDB_Symptoms( DataSet = DataSet, 
                                      Symptom_Vars = Symptom_Vars)
  }
  
  return(DataSet)
  
}


RecodeVariable_Overload <- function (DataSet, Old_Values, New_Values, Variable_Name)
{
  
  NewVal_Vector <- vector(length = nrow(DataSet))
  
  Index <- 1
  for (Value in Old_Values)
  {
    NewVal_Vector[which(DataSet[, Variable_Name] == Value)] <- New_Values[Index]
    Index <- Index+1
  }
  
  
  DataSet[, Variable_Name] <- NewVal_Vector
  
  return(DataSet)
  
}

#####   Function: PreprocessDB_Symptoms
###
##
PreprocessDB_Symptoms <- function (DataSet, Symptom_Vars)
{
  
  if (is.null(Symptom_Vars))
  {
    print("Error when preprocessing database (PreprocessDB_Symptoms in FASS_Base_Preprocess.R): Symptom_Vars not defined")
    return(NULL)
  }
  
  for (Symptom in Symptom_Vars)
  {
    print(paste0("Recoding symptom to numeric variable: ", Symptom))
    DataSet <- RecodeVariable_Overload(DataSet, Old_Values = Symptom_Values, New_Values = SymptomNumeric_Values, Variable_Name=Symptom)
  }
  
  return(DataSet)
  
  
}



