##### FASS_FASS.R
##### Functions for score and grade computation
#####
##### Results:
#####   Score computed from FOOD database (and added to it)
#####   Grade (based on medical criterion) computed from FOOD database (and added to it)
#####   Grade (computed from the score) computed from FOOD database (and added to it)

source(paste0(getwd(), "/FASS_Descriptive_SeverityStudy.R"))


##### FASS_Compute
###   Function that computes oFASS, oFASS3 and nFASS from the provided dataset, incorporating them into the database
##      as new variables. 
##    Inputs:
##      Dataset           - Dataset containing information about patients and their symptoms.
##      Systems           - List of names of the systems in the database.
##      SymptomsBySystem  - List lists of names of the symptoms. Each entry corresponds with one system in the Systems list.
##      GradeSysts        - Grade associated with each system, for oFASS and oFASS3 calculation.
##      ExponentSysts     - List of exponents associated with each system in System, for nFASS calculation.
##      Lambda            - Weight for each symptom in the system.
##    Output:
##      Dataset           - Same dataset provided as input, with the new variables added to it.

FASS_Compute <- function (Dataset, Systems, Symptoms, SymptomsBySystem, GradeSysts, ExponentSysts, SymptomLambdas)
{
  
  nFASS_Var_Loc <- nFASS_Var
  oFASS_Var_Loc <- oFASS_Var
  oFASS3_Var_Loc <- oFASS3_Var
  
  Dataset[, nFASS_Var_Loc] <- Score(Dataset, Systems, Symptoms, SymptomsBySystem, ExponentSysts, SymptomLambdas)
  
  Dataset[, oFASS_Var_Loc] <- GradeMedical(Dataset, Systems, SymptomsBySystem, GradeSysts)
  
  Dataset[, oFASS3_Var_Loc] <- GradeUptoThree(Dataset, oFASS_Var_Loc)
  
  print(paste0(nFASS_Var_Loc, ", ", oFASS_Var_Loc, " and ", oFASS3_Var_Loc, " added to local database"))
  
  return(Dataset)
}

##### FUNCTION: Score
###   Computes the score for the given dataset (and adds the column)
#     Inputs: 
#       Dataset         - Dataset for score computing.
#       Systems         - Vector with the system keys
#       SystemSympts    - Key-list containing the symptom variables for each system
#       ExponentSysts   - Key-list containing the exponents for each system
#     Outputs:
#       Scores          - Vector containing the computed scores
# 


Score <- function(Dataset, Systems, Symptoms, SystemSympts, ExponentSysts, SymptomLambdas)
{
  # Initialize the score vector as a vector of zeros
  Scores <- rep(0, nrow(Dataset))
  
  for (System in Systems)
  {
    SystemIndex <- which(Systems == System)
    
    System_Symptoms <- SystemSympts[[SystemIndex]]
    
    CumLambdas <- rep(0.0, nrow(Dataset))
    
    for (Symptom in System_Symptoms)
    {
      ###   Extract lambda corresponding to symptom
      Lambda <- SymptomLambdas[which(Symptoms == Symptom)]
      
      ###   Accumulate lambdas by reaction reported
      CumLambdas <- as.numeric(unlist(CumLambdas + Lambda*Dataset[ , Symptom]))
      }
    
    # Compute the contribution of the system to the score
    SysScore <- ifelse( CumLambdas == 0.0, 0.0, 2^(ExponentSysts[SystemIndex])*(1+CumLambdas) ) 
    
    Scores <- SysScore + Scores
    
  }
  
  return(Scores)
  
}

nFASS_by_levels <- function(DataSet, Systems, SystemSympts, ExponentSysts, SymptomLambdas, SymptomLevels)
{
  # Initialize the score vector as a vector of zeros
  Scores <- rep(0, nrow(DataSet))
  
  for (System in Systems)
  {
    SystemIndex <- which(Systems == System)
    System_Symptoms <- SystemSympts[[SystemIndex]]
    
    
    CumLambdas <- rep(0, nrow(DataSet))
    for (Symptom in System_Symptoms)
    {
      ###   Select the lambda in depends on the value expressed by the patient
      Patients_Lambdas <- rep(0, length(DataSet[ , Symptom]))
      for (Level in SymptomLevels)
      {
        Index <- which(SymptomLevels == Level)
        Patients_Lambdas <- ifelse(DataSet[, Symptom] == Level, SymptomLambdas(Index), 0)
      }
      
      ###   Accumulate lambdas by reaction reported
      CumLambdas <- CumLambdas + Patients_Lambdas*DataSet[ , Symptom]
    }
    
    # Compute the contribution of the system to the score
    SysScore <- ifelse( CumLambdas == 0.0, 0.0, 2^(ExponentSysts[SystemIndex])*(1+CumLambdas) ) 
    
    Scores <- SysScore + Scores
    
  }
  
  return(Scores)
  
}

##### FUNCTION: GradeMedical
###   Computes the grade of each patient USING MEDICAL CRITERION (most sever system affected determines grade, grade 3 as the 
#       combination of two or more systems of grade 2)       
#
#     Inputs: 
#       DataSet         - Dataset for score computing.
#       Systems         - Vector with the system keys
#       SystemSympts    - Key-list containing the symptom variables for each system
#       GradeSysts      - Key-list containing the grade of each system
#     Outputs:
#       Grades          - Vector containing the computed grades
GradeMedical <- function(DataSet, Systems, SystemSympts, GradeSysts)
{
  
  Grades <- rep(0, nrow(DataSet))
  
  # Vector for handling the grade2-3 special case
  Grade2Systs <- rep(0, nrow(DataSet))
  
  for (System in Systems)
  {
    Index <- which(Systems == System)
    Symptoms <- SystemSympts[[Index]]
    
    # Check if the element is a vector (only one symptom in the system)
    if (is.vector(DataSet[,Symptoms]))
      Counts <- DataSet[,Symptoms]
    else
      Counts <- rowSums(DataSet[,Symptoms])
    
    # Compute the new grade (if the system causes the grade to update
    if (GradeSysts[[Index]] == 2)
    {
      # Special grade 2-3 case
      Grade2Systs <- ifelse(Counts > 0, Grade2Systs+1, Grade2Systs)
      # If 2 or more systems of grade 2 are affected, grade is 3, if only one, grade is 2, if 0, grade is not modified (yet)
      SysGrades <- ifelse(Grade2Systs > 1, 3, ifelse(Grade2Systs == 1, 2, 0))
    }
    else
      SysGrades <- ifelse(Counts > 0, GradeSysts[[Index]], 0)
    
    # Update grades
    Grades <- pmax(Grades, SysGrades)
    
  }
  
  return(Grades)
  
}


GradeUptoThree <- function(DataSet, GradeMedVariable)
{
  GradeShort <- ifelse(DataSet[,GradeMedVariable] == 1, 1, 
                       ifelse(DataSet[,GradeMedVariable] == 2 | DataSet[,GradeMedVariable] == 3, 2, 
                              ifelse(DataSet[,GradeMedVariable] == 4 | DataSet[,GradeMedVariable] == 5, 3, 0)
                       )
  )
  
  
  
  return(GradeShort)
}

FactorizeScores <- function(FitData)
{
  ### Dummy variable for the grade (convert into categorical before regression)
  FitData$oFASS <- factor(FitData$oFASS)
  
  FitData$oFASS3 <- factor(FitData$oFASS3)
  
  return(FitData)
}






